#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Main module
"""
import os
import gtk
import gtk.glade
import gettext

import view
import remarker
import customs
import client
import cloud
import time

#from time import gmtime, strftime

from gettext import gettext as _
gettext.textdomain('hannibal')


class Base(object):

    def __init__(self):
        self.toolbar_visible = True     # toolbar visibility
        self.side_panel_visible = True  # right panel visibility
        self.file_uri = ''              # path to PDF
        self.user = ''                  # user name
        self.email = ''
        self.document_title = None      # document title from meta
        self.document_created = None    # document creation timestamp
        self.document_updated = None    # document update timestamp

        # Fetch UI from XML
        self.xml = gtk.glade.XML(fname="main.glade", root="window_main")
        # Connect any signals for widgets from XML
        self.xml.signal_autoconnect(self)
        # Fetch widgets from XML and coonect signals to proper callbacks
        self.window = self.xml.get_widget("window_main")
        self.statusbar = self.xml.get_widget("statusbar1")
        self.vbox = self.xml.get_widget("vbox1")
        self.toolbar = self.xml.get_widget("toolbar1")
        self.hpaned1 = self.xml.get_widget("hpaned1")
        self.vpaned1 = self.xml.get_widget("vpaned1")
        self.prev_button = self.xml.get_widget("prev_button")
        self.prev_button.connect("clicked", self.on_page_prev)
        self.next_button = self.xml.get_widget("next_button")
        self.next_button.connect("clicked", self.on_page_next)
        self.refresh_button = self.xml.get_widget("toolbutton_refresh")
        self.refresh_button.connect("clicked", self.on_refresh)
        self.refresh_button.set_sensitive(False)
        self.prev_menu_item = self.xml.get_widget("gtk_go_back")
        self.prev_menu_item.connect("activate", self.on_page_prev)
        self.next_menu_item = self.xml.get_widget("gtk_go_forward")
        self.next_menu_item.connect("activate", self.on_page_next)


        self.toolbar_toggle = self.xml.get_widget("toolbar_toggle")
        self.side_panel_toggle = self.xml.get_widget("side_panel_toggle")

        self.window.set_title(_("Hannibal"))
        self.window.set_icon_from_file("icon.png")

        self.page_field = customs.MySpinTool()
        self.page_field.sb.connect("value-changed", self.on_page_change)

        self.scale_field = customs.MyComboBox()
        self.scale_field.cb.connect("changed", self.on_scale_change)
        #self.scale_field.cb.append_text("Уместить в окне")
        #self.scale_field.cb.append_text("По ширине страницы")
        self.scale_field.cb.append_text("100%")
        self.scale_field.cb.append_text("200%")
        self.scale_field.cb.append_text("300%")
        self.view = view.View(self)
        self.scale_field.cb.set_active(0)


        self.toolbar.add(self.page_field)
        self.toolbar.add(gtk.SeparatorToolItem())
        self.toolbar.add(self.scale_field)


        self.left_sw = self.xml.get_widget("left_sw")

        self.rem = remarker.Remarker(self)
        self.left_sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.left_sw.add_with_viewport(self.view)
        self.hpaned = self.xml.get_widget("hpaned1")
        self.hpaned.pack2(self.rem, resize=False, shrink=False)

        self.toolbar_toggle.set_active(self.toolbar_visible)
        self.toolbar_toggle.toggled()
        self.toolbar.set_sensitive(False)

        self.side_panel_toggle.set_active(self.side_panel_visible)
        self.side_panel_toggle.toggled()
        self.window.show_all()
        # Inactivate some widgets
        #self.menu = self.xml.get_widget("menubar1")
        self.menu_file_export = self.xml.get_widget("export")
        self.menu_file_import = self.xml.get_widget("import")
        self.menu_file_gtk_go_back = self.xml.get_widget("gtk_go_back")
        self.menu_file_gtk_go_forward = self.xml.get_widget("gtk_go_forward")
        
        self.menu_file_export.set_sensitive(False)
        self.menu_file_import.set_sensitive(False)
        self.menu_file_gtk_go_back.set_sensitive(False)
        self.menu_file_gtk_go_forward.set_sensitive(False)
        #self.open_file('/home/elf/forge/python/hannibal/81.pdf')
        #self.open_file('/home/elf/forge/python/hannibal/test-page-labels.pdf')
        #self.on_import(self)
        self.get_preferences()
        if not self.user:
            self.set_preferences()
        self.rem.get_clouds()

    ###########################################################################
    def open_file(self, f):
        """Load file with some checks."""
        if f is not None and os.path.isfile(f):
            self.view.get_from_file(f)
            self.page_field.sb.set_range(1, self.view.n_pages)
            self.page_field.sb.set_increments(1, 1)
            # Prepare some widgets for new document
            self.page_field.sb.set_value(1)
            # Some widgets should be sensitive now
            self.menu_file_export.set_sensitive(True)
            self.menu_file_import.set_sensitive(True)

    ###########################################################################
    def get_preferences(self):
        """Get preferences."""
        try:
            # Try to get data from config file
            f = open(".config", "r")
            self.user, self.email = eval(f.read())
            f.close()
        except:
            print "\t[w] Cannot open a file"

    ###########################################################################
    def set_preferences(self):
        """Set preferences."""
        p = customs.Preferences({"name":self.user, "email":self.email})
        r = p.run()
        if r == gtk.RESPONSE_CLOSE:
            name = p.entry_user_name.get_text()
            email = p.entry_user_email.get_text()
            if name:
                self.user = name
                self.email = ''.join(email)
                try:
                    f = open(".config", "w")
                    f.write(str([self.user, self.email]))
                    f.close()
                    #print "\t[*] Saved."
                except:
                    print "\t[e] Cannot save prefs."
        p.destroy()


    ###########################################################################
    # Callback methods
    ###########################################################################

    ###########################################################################
    def on_realize(self, event):
        """Called when "realized" signal was received."""
        self.gdkw = self.dwg.window
        self.gc = self.gdkw.new_gc()

    ###########################################################################
    def on_expose(self, widget, event):
        """Called when "expose" signal was received.

        Usually it is raised when widget should be visually refreshed.
        """
        cr = widget.window.cairo_create()
        cr.set_source_rgb(1, 1, 1)

        if self.scale != 1:
            cr.scale(self.scale, self.scale)

        if self.buf is not None:
            self.get_buffer()

        self.put_buffer()

    ###########################################################################
    def on_toolbar_toggled(self, widget):
        """Called when toolbar visibility is toggled.

        It should be connected to proper menu item.
        """
        # Get new value
        self.toolbar_visible = widget.get_active()

        # If True — toolbar is visible
        if self.toolbar_visible:
            # Check whether toolbar has a parent (is it inside any container?).
            # If not — pack it into container at the 1st position.
            if not self.toolbar.get_parent():
                self.vbox.pack_start(self.toolbar,
                                    expand=False,
                                    fill=True,
                                    padding=0)
                self.vbox.reorder_child(child=self.toolbar, position=1)

        # Else if toolbar is already in container it has to be hidden.
        # It is done by removing toolber from container.
        elif self.toolbar.get_parent():
            self.vbox.remove(self.toolbar)

    ###########################################################################
    def on_side_panel_toggled(self, widget):
        """Called when side panel visibility is toggled.

        It should be connected to proper menu item.
        """
        # Get new value
        self.side_panel_visible = widget.get_active()

        # If True — side panel is visible
        if self.side_panel_visible:
            # Check whether panel has a parent (is it inside any container?).
            # If not — pack it into container at the 1st position.
            if not self.rem.get_parent():
                self.hpaned1.pack2(self.rem, resize=False, shrink=True)

        # Else if panel is already in container it has to be hidden.
        # It is done by removing panel from container.
        elif self.rem.get_parent():
            self.hpaned1.remove(self.rem)

    ###########################################################################
    def on_scale_change(self, widget):
        """Called on scale change."""
        self.view.set_scale(self.scale_field.cb.get_active())

    ###########################################################################
    def on_page_change(self, widget):
        """Called on page change."""
        cp = widget.get_value_as_int()
        self.view.set_page_num(cp-1)

        if self.view.n_pages > cp:
            self.next_button.set_sensitive(True)
            self.menu_file_gtk_go_forward.set_sensitive(True)
        else:
            self.next_button.set_sensitive(False)
            self.menu_file_gtk_go_forward.set_sensitive(False)

        if cp <= 1:
            self.prev_button.set_sensitive(False)
            self.menu_file_gtk_go_back.set_sensitive(False)
        else:
            self.prev_button.set_sensitive(True)
            self.menu_file_gtk_go_back.set_sensitive(True)

    ###########################################################################
    def on_page_next(self, widget):
        """Callback for "Next" toolbutton clicked.

        Page number should be incremented.
        """
        self.view.next_page()
        self.page_field.sb.spin(gtk.SPIN_STEP_FORWARD)

    ###########################################################################
    def on_page_prev(self, widget):
        """Callback for "Prev" toolbutton clicked.

        Page number should be decremented.
        """
        self.view.prev_page()
        self.page_field.sb.spin(gtk.SPIN_STEP_BACKWARD)

    ###########################################################################
    def on_export(self, widget):
        """Save remarks to file."""
        dlg = gtk.FileChooserDialog(title=None,
                                    parent=self.window,
                                    action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                    buttons=None,
                                    backend=None)
        dlg.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        dlg.add_button(gtk.STOCK_SAVE, 1983)

        dlg.set_icon(self.window.get_icon())
        dlg.set_title(_("Export Remarks..."))
        dlg.set_default_response(1983)
        # Add FileFilter: allows to filter file types to be opened
        filt = gtk.FileFilter()
        #filt.add_mime_type("application/pdf")
        filt.set_name("*.hrx")
        filt.add_pattern("*.hrx")
        dlg.add_filter(filt)

        filt = gtk.FileFilter()
        filt.set_name("*.*")
        filt.add_pattern("*.*")
        dlg.add_filter(filt)
        dlg.set_current_name('remarks_'+self.user+'.hrx')

        res = dlg.run()

        if res == gtk.RESPONSE_CANCEL:
            pass

        elif res == gtk.RESPONSE_DELETE_EVENT:
            pass

        elif res == 1983:
            f = dlg.get_filename()
            if f is not None:
                self.rem.export_to_hrx(f)
#                try:
#                except:
#                    print "\t[e] Cannot save dump."
        dlg.destroy()
            
    ###########################################################################
    def on_import(self, widget):
        """Get file for import."""
        dlg = gtk.FileChooserDialog(title=None,
                                    parent=self.window,
                                    action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                    buttons=None,
                                    backend=None)
        dlg.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        dlg.add_button(gtk.STOCK_OPEN, 1983)

        dlg.set_icon(self.window.get_icon())
        dlg.set_title(_("Import Remarks..."))
        dlg.set_default_response(1983)
        # Add FileFilter: allows to filter file types to be opened
        filt = gtk.FileFilter()
        #filt.add_mime_type("application/pdf")
        filt.set_name("*.hrx")
        filt.add_pattern("*.hrx")
        dlg.add_filter(filt)
        filt = gtk.FileFilter()
        filt.set_name("*.*")
        filt.add_pattern("*.*")
        dlg.add_filter(filt)

        res = dlg.run()
        f = dlg.get_filename()
        dlg.destroy()
        if res == gtk.RESPONSE_CANCEL:
            pass
        elif res == gtk.RESPONSE_DELETE_EVENT:
            pass
        elif res == 1983:
            if f is not None and os.path.isfile(f):
#                try:
                    self.rem.import_from_hrx(f)
#                except:
#                    print "[e] Can't open a file!"

    ###########################################################################
    def on_refresh(self, widget):
        """Callback for "Refresh" toolbutton clicked.

        It should perform some requests to server.
        """
        pass

    ###########################################################################
    def on_gtk_open(self, widget):
        """Callback for "File->Open" activation."""
        dlg = gtk.FileChooserDialog(title=None,
                                    parent=self.window,
                                    action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                    buttons=None,
                                    backend=None)

        dlg.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        dlg.add_button(gtk.STOCK_OPEN, 1983)

        dlg.set_icon(self.window.get_icon())
        dlg.set_title(_("Open File"))
        dlg.set_default_response(1983)

        filt = gtk.FileFilter()
        filt.add_mime_type("application/pdf")
        filt.set_name("*.pdf")
        dlg.add_filter(filt)

        res = dlg.run()

        if res == gtk.RESPONSE_CANCEL:
            print "Cancel"

        elif res == gtk.RESPONSE_DELETE_EVENT:
            print "Closed"

        elif res == 1983:
            f = dlg.get_filename()
            # FIXME: CHECK must be done!!!
            try:
                self.open_file(f)
            except:
                print "[e] Can't open a file!"

        dlg.destroy()

    ###########################################################################
    def on_gtk_pref(self, widget):
        """Callback for "Edit->Preferences" activation."""
        self.set_preferences()

    ###########################################################################
    def on_gtk_about(self, widget):
        """Callback for "Help->About" activation."""
        dlg = gtk.AboutDialog()
        dlg.set_program_name("Hannibal")
        dlg.set_name("Hannibal")
        dlg.set_version("0.1")
        authors = []
        authors.append("Oleg Kulik <helh-saintman@yandex.ru>")
        authors.append("Valentina Mukhamedzhanova")
        authors.append("Victor Nikulshin")
        dlg.set_authors(authors)
        dlg.set_copyright("Copyright © 2010 Oleg Kulik, Valentina Mukhamedzhanova,\nVictor Nikulshin")
        dlg.run()
        dlg.destroy()

    ###########################################################################
    def check_for_unsaved(self):
        if self.rem.is_changed:
            dlg = gtk.MessageDialog(parent=None,
                                    flags=0,
                                    type=gtk.MESSAGE_WARNING,
                                    buttons=gtk.BUTTONS_YES_NO,
                                    message_format=None)
            dlg.set_markup(_("<b>There are some unsaved changes.</b>"))
            dlg.set_title(_("Quit"))
            dlg.format_secondary_text(_("Quit anyway?"))
            r = dlg.run()
            dlg.destroy()
            if r == gtk.RESPONSE_YES:
                return True
            if r == gtk.RESPONSE_NO:
                return False
        return True

    ###########################################################################
    def delete_event(self, widget, event):
        """Close application and its window."""
        if self.check_for_unsaved():
            return False
        return True

    ###########################################################################
    def gtk_quit_event(self, widget):
        if self.check_for_unsaved():
            gtk.main_quit()

    ###########################################################################
    def destroy(self, widget):
        gtk.main_quit()

    ###########################################################################
    def main(self):
        gtk.main()

if __name__ == "__main__":
    base = Base()
    base.main()

# end of main.py
